﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Optimization.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.AlgorithmBehavior.Analyzers {
  [Item("AfterCrossoverCombinedOperator", "An operator that contains all operators that need to be executed after crossover.")]
  [StorableClass]
  public class AfterCrossoverCombinedOperator : CombinedOperator {
    [Storable]
    protected CrossoverPerformanceAnalyzer cxAnalyzer;

    [Storable]
    protected SolutionToPopulationAnalyzer solToPopAnalyzer;

    [Storable]
    protected VariableDuplicator varDuplicator;

    public ILookupParameter<IEvaluator> EvaluatorParameter {
      get { return ((LookupParameter<IEvaluator>)Parameters["Evaluator"]); }
    }
    public IValueParameter<SingleObjectiveSolutionSimilarityCalculator> SimilarityCalculatorParameter {
      get { return (IValueParameter<SingleObjectiveSolutionSimilarityCalculator>)Parameters["SimilarityCalculator"]; }
    }

    [StorableConstructor]
    protected AfterCrossoverCombinedOperator(bool deserializing) : base(deserializing) { }
    protected AfterCrossoverCombinedOperator(AfterCrossoverCombinedOperator original, Cloner cloner)
      : base(original, cloner) {
      cxAnalyzer = original.cxAnalyzer != null ? (CrossoverPerformanceAnalyzer)original.cxAnalyzer.Clone(cloner) : null;
      varDuplicator = original.varDuplicator != null ? (VariableDuplicator)original.cxAnalyzer.Clone(cloner) : null;
      solToPopAnalyzer = original.solToPopAnalyzer != null ? (SolutionToPopulationAnalyzer)original.solToPopAnalyzer.Clone(cloner) : null;
    }

    public AfterCrossoverCombinedOperator()
      : base() {
      Parameters.Add(new LookupParameter<IEvaluator>("Evaluator", "The operator which is used to evaluate new solutions."));
      Parameters.Add(new ValueParameter<SingleObjectiveSolutionSimilarityCalculator>("SimilarityCalculator"));

      SimilarityCalculatorParameter.ValueChanged += new System.EventHandler(SimilarityCalculatorParameter_ValueChanged);
    }

    protected virtual void SimilarityCalculatorParameter_ValueChanged(object sender, System.EventArgs e) {
      SimilarityCalculatorParameter.Value.QualityVariableName = "TSPTourLength";
      SimilarityCalculatorParameter.Value.SolutionVariableName = "TSPTour";
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new AfterCrossoverCombinedOperator(this, cloner);
    }

    public override void InitializeOperators() {
      cxAnalyzer = new CrossoverPerformanceAnalyzer();
      cxAnalyzer.SimilarityCalculatorParameter.Value = SimilarityCalculatorParameter.Value;
      varDuplicator = new VariableDuplicator();
      solToPopAnalyzer = new SolutionToPopulationAnalyzer();
      solToPopAnalyzer.ChartPostfixParameter.Value = new Data.StringValue("after Crossover");

      Operators.Add(EvaluatorParameter.ActualValue);
      Operators.Add(cxAnalyzer);
      Operators.Add(varDuplicator);
      Operators.Add(solToPopAnalyzer);
    }
  }
}
